import gzip
import json
import os
import unidecode
import re
import requests


def _name(card, name=None):
    name = name or card["name"]
    name = unidecode.unidecode(name).lower()
    name = re.sub(r"[^a-zA-Z0-9]", "", name)
    return name


def _get(card, set, name=None):
    return {
        "key": _name(card),
        "name": (name or card["name"]),
        "date": set["releaseDate"],
        "id": card["identifiers"]["scryfallId"],
    }


def _download(url):
    file = url.split("/")[-1]
    response = requests.get(url, stream=True)
    with open(file, "wb") as file:
        response = requests.get(url)
        file.write(response.content)


def _delete(file):
    if os.path.isfile(file):
        os.remove(file)
    else:
        print(r"{file} not found")


SKIP_TYPES = [
    "promo",
    "funny",
    "box",
    "memorabilia",
    "alchemy",
    "masterpiece",
    "archenemy",
    "starter",
    "token",
    "arsenal",
    "duel_deck",
    "from_the_vault",
    "treasure_chest",
    "vanguard",
    "premium_deck",
    "spellbook",
]


def build():
    # Download SetList.json.gz
    _download("https://mtgjson.com/api/v5/SetList.json.gz")
    sets = json.load(gzip.open("SetList.json.gz"))
    # Download AllPrintings.json.gz
    _download("https://mtgjson.com/api/v5/AllPrintings.json.gz")
    prints = json.load(gzip.open("AllPrintings.json.gz"))

    library = {}
    for set in sets["data"]:
        if set["type"] not in SKIP_TYPES:
            for card in prints["data"][set["code"]]["cards"]:
                # Adding it to library
                if _name(card) not in library:
                    library[_name(card)] = _get(card, set)
                if _name(card) in library:
                    # If newer version exists
                    if set["releaseDate"] > library[_name(card)]["date"]:
                        library[_name(card)] = _get(card, set)

                # Cards may have another face : name_face_a / name_face_b
                # Cards with other faces have a "//" separator
                if "//" in card["name"]:
                    (face_a, face_b) = card["name"].split(" // ")
                    # Adding faces to main card and every parts
                    faces = [_name(card, face_a), _name(card, face_b)]
                    library[_name(card)]["faces"] = faces
                    if _name(card, face_a) not in library:
                        library[_name(card, face_a)] = _get(card, set, face_a)
                        library[_name(card, face_a)]["faces"] = faces
                    if _name(card, face_b) not in library:
                        library[_name(card, face_b)] = _get(card, set, face_b)
                        library[_name(card, face_b)]["faces"] = faces

    # Deleting downloaded files
    _delete("AllPrintings.json.gz")
    _delete("SetList.json.gz")

    try:
        with open("library.json", "w", encoding="utf-8") as file:
            json.dump(library, file, ensure_ascii=False, indent=4)
    except IOError:
        pass

    # Returning constructed library
    return library


if __name__ == "__main__":
    build()



import re
import unidecode
import requests
import os
import json
import pkg_resources
import copy

import urllib.parse
from urllib.parse import urlparse

import flask
from flask import make_response, request, render_template
import jinja2.exceptions

from . import config
from . import navigation
from . import card_list


version = pkg_resources.Environment()["barrins-codex"][0].version
if version[-5:] == ".dev0":  # To pass tests on commits during development
    version = version[:-5]
    version = version[:-1] + str(int(version[-1]) - 1)

app = flask.Flask(__name__, template_folder="templates", static_folder="static")
app.jinja_env.policies["ext.i18n.trimmed"] = True
config.configure_app(app)

if os.path.isfile("library.json"):
    with open("library.json", "r", encoding="utf-8") as file:
        CARDS = json.load(file)
else:
    CARDS = card_list.build()

BASE_CONTEXT = {
    "version": version,
}


def main():
    # print(navigation.HELPER)
    app.run()


# Defining Errors
@app.errorhandler(jinja2.exceptions.TemplateNotFound)
@app.errorhandler(404)
def page_not_found(error):
    return flask.render_template("404.html"), 404


# Static file redirection
@app.route("/favicon.ico")
def favicon():
    return flask.redirect(flask.url_for("static", filename="img/favicon.ico"))


# Serve robots static file
@app.route("/robots.txt")
def static_from_root():
    return flask.send_from_directory(app.static_folder, flask.request.path[1:])


# Serve sitemap template
# code used from https://gist.github.com/Julian-Nash/aa3041b47183176ca9ff81c8382b655a
@app.route("/sitemap.xml")
def sitemap():
    host_components = urlparse(request.host_url)
    host_base = host_components.scheme + "://" + host_components.netloc

    urls = [host_base + p["self"].url for p in navigation.HELPER.values()]

    xml_sitemap = render_template("sitemap.xml", urls=urls, host_base=host_base)
    response = make_response(xml_sitemap)
    response.headers["Content-Type"] = "application/xml"

    return response


# Serve webfonts
@app.route("/webfonts/<path:font>")
def static_fonts(font=None):
    return flask.redirect(flask.url_for("static", filename=f"fonts/{font}"))


# Default route
@app.route("/")
@app.route("/<path:page>")
def index(page=None):
    redirect = False
    if not page:
        page = "index.html"
        redirect = True
    page = "/" + page
    if redirect:
        return flask.redirect(page, 301)

    context = copy.copy(BASE_CONTEXT)
    return flask.render_template(page, **context)


def _build_url(page, _anchor=None, **params):
    url = page.url
    if params:
        url += "?" + urllib.parse.urlencode(params)
    if _anchor:
        url += "#" + _anchor
    return url


def _link(page, name=None, _anchor=None, _class=None, **params):
    if not page or not page.url:
        return ""
    name = name or page.name
    url = _build_url(page, _anchor, **params)
    if _class:
        # _class = f"class='{_class} text-reset'"
        _class = f"class='{_class}'"
    else:
        _class = "class='text-reset'"
    return flask.Markup(f'<a {_class} href="{url}">{name}</a>')


@app.context_processor
def linker():
    path = flask.request.path
    if path[-11:] == "/index.html":
        path = path[:-11]
    if path[-5:] == ".html":
        path = path[:-5]
    if path[-1:] == "/":
        path = path[:-1]

    def _url(page, _anchor=None, **params):
        return _build_url(
            navigation.HELPER.get(page, {}).get("self"), _anchor=_anchor, **params
        )

    def link(page, name=None, _anchor=None, **params):
        return _link(
            navigation.HELPER.get(page, {}).get("self"),
            name=name,
            _anchor=_anchor,
            **params,
        )

    def name(page, name=None):
        return name or navigation.HELPER.get(page, {}).get("self").name

    def top():
        return _link(navigation.HELPER.get(path, {}).get("top"), _class="text-reset")

    def next():
        return _link(
            navigation.HELPER.get(path, {}).get("next"), _class="next text-reset"
        )

    def prev():
        return _link(
            navigation.HELPER.get(path, {}).get("prev"), _class="prev text-reset"
        )

    def external(url, name, _class=None):
        if _class:
            _class = f"class='{_class}'"
        else:
            _class = "class='text-reset text-decoration-underline'"
        return flask.Markup(
            f'<a {_class} target="_blank" rel="noreferrer" href="{url}">{name}</a>'
        )

    def title():
        try:
            if navigation.HELPER.get(path, {}).get("self").path != "":
                name = navigation.HELPER.get(path, {}).get("self").name
                return f"Barrin's Codex - {name}"
        except AttributeError:
            pass
        return "Barrin's Codex"

    def page_name():
        try:
            if navigation.HELPER.get(path, {}).get("self").path != "":
                name = navigation.HELPER.get(path, {}).get("self").name
                return f"{name}"
        except AttributeError:
            pass
        return "Barrin's Codex"

    return dict(
        url=_url,
        link=link,
        title=title,
        name=name,
        page_name=page_name,
        top=top,
        next=next,
        prev=prev,
        external=external,
    )


@app.context_processor
def display_card():
    def get_card(name, firstprint=True):
        fuzzy = re.sub(r"[^\w\s]", "", name)
        fuzzy = re.sub(r"\s+", "-", fuzzy)
        url = f"https://api.scryfall.com/cards/search?q={fuzzy}"
        if firstprint:
            url = url + "+is%3Afirstprint"
        r = requests.get(url)
        return r.json()

    def _name(name):
        name = name
        name = unidecode.unidecode(name).lower()
        name = re.sub(r"[^a-zA-Z0-9]", "", name)
        return name

    def card_name_from_page(name):
        page_name = navigation.HELPER.get(name, {}).get("self").name
        if "❌" in page_name:
            page_name = page_name[2:]
        return page_name

    def img_crop(name, front=True):
        card = CARDS[_name(name)]
        if "faces" in card:
            if not front:
                card = get_card(name)["data"][0]
                return card["card_faces"][1]["image_uris"]["art_crops"]
            return (
                "https://api.scryfall.com/cards/"
                + f"{CARDS[card['faces'][0]]['id']}"
                + "?format=image&version=art_crop"
            )
        return (
            "https://api.scryfall.com/cards/"
            + f"{card['id']}?format=image&version=art_crop"
        )

    def img_card(name, firstprint=True, front=True):
        card = CARDS[_name(name)]
        if "faces" in card:
            if not front:
                card = get_card(name, firstprint)["data"][0]
                return card["card_faces"][1]["image_uris"]["png"]
            card = get_card(name, firstprint)["data"][0]
            return card["card_faces"][0]["image_uris"]["png"]
        if firstprint:
            card = get_card(name, firstprint)["data"][0]
            return card["image_uris"]["png"]
        return f"https://api.scryfall.com/cards/{card['id']}?format=image"

    def card_link(name):
        return flask.Markup(
            '<a target="_blank" class="text-reset text-decoration-underline" '
            + 'rel="noreferrer" href="'
            + get_card(name)["data"][0]["scryfall_uri"]
            + '">'
            + name
            + "</a>"
        )

    return dict(
        deck_name=card_name_from_page,
        img_crop=img_crop,
        img_card=img_card,
        card_link=card_link,
    )


@app.context_processor
def display_match():
    def match_name(page):
        try:
            if navigation.HELPER.get(page, {}).get("self").path != "":
                name = navigation.HELPER.get(page, {}).get("self").name
                return name.split(" ", 1)[1]
        except AttributeError:
            pass
        return "Barrin's Codex"

    return dict(match_name=match_name)



import collections
import re
import unidecode


Page = collections.namedtuple("Page", ["name", "path", "url"])


class Nav:
    def __init__(self, name, index=False, children=None):
        self.name = name
        self.index = index
        self.children = children or []

    def page(self, path):
        res = unidecode.unidecode(str(self.name))
        res = re.sub(r"[^\sa-zA-Z0-9]", "", res).lower().strip()
        res = re.sub(r"\s+", "-", res)
        if res == "home":
            res = path
        else:
            res = path + "/" + res
        if not self.children:
            url = res + ".html"
        elif self.index:
            url = res + "/index.html"
        else:
            url = None
        return Page(self.name, res, url)

    def walk(self, path=None, top=None, ante=None, post=None):
        page = self.page(path or "")
        if not self.children or self.index:
            yield (page.path, {"self": page, "top": top, "prev": ante, "next": post})
        if self.index:
            top = page
        for i, child in enumerate(self.children):
            if i > 0:
                ante = self.children[i - 1].page(page.path)
            else:
                ante = None
            if i < len(self.children) - 1:
                post = self.children[i + 1].page(page.path)
            else:
                post = None
            yield from child.walk(path=page.path, top=top, ante=ante, post=post)


STRUCTURE = Nav(
    # TRANSLATORS: please abide by MTG translation choices for game terms when possible.
    "Home",
    index=True,
    children=[
        Nav(
            "🧍 Pilotes",
            index=True,
            children=[
                Nav("Anaël « Aliquanto »"),
                Nav("Bastien « threem »"),
                Nav("Damien « eventwin »"),
                Nav("Domino"),
                Nav("Félix « Midas »"),
                Nav("Florian « Trotte »"),
                Nav("Grégoire « Koraysh »"),
                Nav("Louis « Imperia86 »"),
                Nav("Martin « Spigushe »"),
                Nav("Apparitions"),
            ],
        ),
        Nav(
            "📝 Decks",
            index=True,
            children=[
                Nav("Adeliz, the Cinder Wind"),
                Nav("Aminatou, the Fateshifter"),
                Nav("Azusa, Lost but Seeking"),
                Nav("Bjorna-Wernog"),
                Nav("Dennick, Pious Apprentice"),
                Nav("Doran, the Siege Tower"),
                Nav("Dragonlord Ojutai"),
                Nav("Elmar-Hargilde"),
                Nav("Elmar-Sophina"),
                Nav("Elminster"),
                Nav("Esika, God of the Tree"),
                Nav("Grist, the Hunger Tide"),
                Nav("Hinata, Dawn-Crowned"),
                Nav("Hogaak, Arisen Necropolis"),
                Nav("Isamaru, Hound of Konda"),
                Nav("Ishai-Tevesh"),
                Nav("Jori En, Ruin Diver"),
                Nav("Juri, Master of the Revue"),
                Nav("Kari Zev, Skyship Raider"),
                Nav("Karlov of the Ghost Council"),
                Nav("Killian, Ink Duelist"),
                Nav("Kinnan, Bonder Prodigy"),
                Nav("Klothys, God of Destiny"),
                Nav("Light-Paws, Emperor's Voice"),
                Nav("Livio-Malcolm"),
                Nav("Livio-Prava"),
                Nav("Maelstrom Wanderer"),
                Nav("Marath, Will of the Wild"),
                Nav("Miara-Tevesh"),
                Nav("Minsc, Beloved Ranger"),
                Nav("❌ Minsc&Boo, Timeless Heroes"),
                Nav("Niv-Mizzet Reborn"),
                Nav("Prossh, Skyraider of Kher"),
                Nav("Queen Marchesa"),
                Nav("Raffine, Scheming Seer"),
                Nav("Sai, Master Thopterist"),
                Nav("Saskia the Unyielding"),
                Nav("❌ Shorikai, Genesis Engine"),
                Nav("Sygg, River Cutthroat"),
                Nav("Sythis, Harvest's Hand"),
                Nav("The Beamtown Bullies"),
                Nav("The Gitrog Monster"),
                Nav("The Reality Chip"),
                Nav("Tiamat"),
                Nav("Tivit, Seller of Secrets"),
                Nav("Wilson, Refined Grizzly"),
            ],
        ),
        Nav(
            "⏯️ Matchs",
            index=True,
            children=[
                Nav("1. Adeliz vs Kinnan"),
                Nav("2. Kari Zev vs Saskia"),
                Nav("3. Isamaru vs Gitrog"),
                Nav("4. Livio-Malcolm vs Niv 5C"),
                Nav("5. Jori En vs Sygg"),
                Nav("6. Aminatou vs Klothys"),
                Nav("7. Livio-Prava vs Ojutai"),
                Nav("8. Esika vs Niv 5C"),
                Nav("9. Miara-Tevesh vs Prossh"),
                Nav("10. Kari Zev vs Marath"),
                Nav("11. Doran vs Marchesa"),
                Nav("12. Light Paws vs Ishai-Tevesh"),
                Nav("13. Bjorna-Wernog vs Grist"),
                Nav("14. Minsc vs ❌ Shorikai"),
                Nav("15. Dennick vs Hinata"),
                Nav("16. Aminatou vs Tiamat"),
                Nav("17. Grist vs Sythis"),
                Nav("18. Kari Zev vs Killian [Budget]"),
                Nav("19. Elmar-Sophina vs ❌ Shorikai"),
                Nav("20. Juri vs Sai"),
                Nav("21. Aminatou vs Beamtown"),
                Nav("22. Raffine vs Tivit"),
                Nav("23. Karlov vs Reality Chip"),
                Nav("24. Grist vs ❌ Minsc&Boo"),
                Nav("25. Elminster vs Wilson"),
                Nav("26. Hogaak vs ❌ Minsc&Boo"),
                Nav("27. Jori En vs Maelstrom Wanderer"),
                Nav("28. Azusa vs Elmar-Hargilde"),
            ],
        ),
    ],
)


HELPER = dict(STRUCTURE.walk())
